#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Problems.DataAnalysis;

namespace HeuristicLab.Algorithms.DataAnalysis {
  /// <summary>
  /// Represents a support vector solution for a regression problem which can be visualized in the GUI.
  /// </summary>
  [Item("SupportVectorRegressionSolution", "Represents a support vector solution for a regression problem which can be visualized in the GUI.")]
  [StorableClass]
  public sealed class SupportVectorRegressionSolution : RegressionSolution {

    public new SupportVectorRegressionModel Model {
      get { return (SupportVectorRegressionModel)base.Model; }
    }

    //IRegressionModel IRegressionSolution.Model {
    //  get { return model; }
    //}
    //IDataAnalysisModel IDataAnalysisSolution.Model {
    //  get { return model; }
    //}

    //[Storable]
    //private IRegressionProblemData problemData;
    //public IRegressionProblemData ProblemData {
    //  get { return problemData; }
    //}
    //IDataAnalysisProblemData IDataAnalysisSolution.ProblemData {
    //  get { return ProblemData; }
    //}

    [Storable]
    private double lowerEstimationLimit;
    public double LowerEstimationLimit {
      get { return lowerEstimationLimit; }
    }

    [Storable]
    private double upperEstimationLimit;
    public double UpperEstimationLimit {
      get { return upperEstimationLimit; }
    }

    private List<string> inputVariables;
    [Storable]
    private IEnumerable<string> InputVariablesStorable {
      get { return inputVariables; }
      set { inputVariables = new List<string>(value); }
    }


    //public event EventHandler ModelChanged;

    //public event EventHandler ProblemDataChanged;

    public Dataset SupportVectors {
      get { return CalculateSupportVectors(); }
    }

    //private List<double> estimatedValues;

    [StorableConstructor]
    private SupportVectorRegressionSolution(bool deserializing) : base(deserializing) { }
    private SupportVectorRegressionSolution(SupportVectorRegressionSolution original, Cloner cloner)
      : base(original, cloner) {
      //problemData = cloner.Clone(original.problemData);
      //model = cloner.Clone(original.model);
      inputVariables = new List<string>(original.inputVariables);
      lowerEstimationLimit = original.lowerEstimationLimit;
      upperEstimationLimit = original.upperEstimationLimit;
    }
    public SupportVectorRegressionSolution(SupportVectorRegressionModel model, IRegressionProblemData problemData, IEnumerable<string> inputVariables, double lowerEstimationLimit, double upperEstimationLimit)
      : base(model, problemData) {
      //this.problemData = problemData;
      //this.model = model;
      this.inputVariables = new List<string>(inputVariables);
      this.lowerEstimationLimit = lowerEstimationLimit;
      this.upperEstimationLimit = upperEstimationLimit;
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new SupportVectorRegressionSolution(this, cloner);
    }

    protected override void OnProblemDataChanged(EventArgs e) {
      Model.Model.SupportVectorIndizes = new int[0];
      base.OnProblemDataChanged(e);
    }

    //public IEnumerable<double> GetEstimatedValues(IEnumerable<int> rows) {
    //  if (estimatedValues == null) RecalculateEstimatedValues();
    //  foreach (int row in rows)
    //    yield return estimatedValues[row];
    //}

    //public override IEnumerable<double> EstimatedValues {
    //  get {
    //    if (estimatedValues == null) RecalculateEstimatedValues();
    //    return estimatedValues;
    //  }
    //}

    //public override IEnumerable<double> EstimatedTrainingValues {
    //  get {
    //    return GetEstimatedValues(ProblemData.TrainingIndizes);
    //  }
    //}

    //public override IEnumerable<double> EstimatedTestValues {
    //  get {
    //    return GetEstimatedValues(ProblemData.TestIndizes);
    //  }
    //}

    private Dataset CalculateSupportVectors() {
      if (Model.Model.SupportVectorIndizes.Length == 0)
        return new Dataset(new List<string>(), new double[0, 0]);

      double[,] data = new double[Model.Model.SupportVectorIndizes.Length, ProblemData.Dataset.Columns];
      for (int i = 0; i < Model.Model.SupportVectorIndizes.Length; i++) {
        for (int column = 0; column < ProblemData.Dataset.Columns; column++)
          data[i, column] = ProblemData.Dataset[Model.Model.SupportVectorIndizes[i], column];
      }
      return new Dataset(ProblemData.Dataset.VariableNames, data);
    }

    //protected override void RecalculateEstimatedValues() {
    //  Dataset dataset = problemData.Dataset;
    //  string targetVariable = problemData.TargetVariable;
    //  IEnumerable<string> allowedInputVariables = problemData.InputVariables.CheckedItems.Select(x => x.Value);
    //  int start = 0;
    //  int end = dataset.Rows;
    //  IEnumerable<int> rows = Enumerable.Range(start, end - start);
    //  SVM.Problem problem = SupportVectorMachineUtil.CreateSvmProblem(dataset, targetVariable, allowedInputVariables, rows);
    //  SVM.Problem scaledProblem = SVM.Scaling.Scale(Model.RangeTransform, problem);

    //  estimatedValues = (from row in Enumerable.Range(0, scaledProblem.Count)
    //                     let prediction = SVM.Prediction.Predict(Model.Model, scaledProblem.X[row])
    //                     let boundedX = Math.Min(UpperEstimationLimit, Math.Max(LowerEstimationLimit, prediction))
    //                     select double.IsNaN(boundedX) ? UpperEstimationLimit : boundedX).ToList();
    //}
  }
}
